const webpack = require("webpack");
const path = require("path");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const { CleanWebpackPlugin } = require("clean-webpack-plugin");
const ProgressBarPlugin = require("progress-bar-webpack-plugin");
const CopyWebpackPlugin = require("copy-webpack-plugin");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");

const info = {
  TITLE: "To-Do List",
};

const config = {
  mode: "development",
  devtool: "source-map",

  devServer: {
    static: path.join(__dirname, "../dist"),
    port: 8080,
    host: "0.0.0.0",
    historyApiFallback: true,
  },

  module: {
    rules: [
      {
        test: /\.css$/,
        use: [MiniCssExtractPlugin.loader, "css-loader"],
      },
      {
        test: /\.(ico|png)$/i,
        use: ["file-loader"],
      },
      {
        test: /.jsx$/,
        exclude: /node_modules/,
        use: {
          loader: "babel-loader",
          options: {
            targets: "defaults",
            presets: [["@babel/preset-env"], ["@babel/preset-react"]],
          },
        },
      },
    ],
  },

  resolve: {
    extensions: [".js", ".jsx", ".json"],
  },

  entry: {
    main: "./src/index.jsx",
  },

  output: {
    filename: "[name].[chunkhash].js",
    path: path.resolve(__dirname, "../dist"),
    publicPath: "/",
  },

  plugins: [
    new ProgressBarPlugin({ width: 80 }),
    new CleanWebpackPlugin(),
    new CopyWebpackPlugin(["src/img/*"]),
    new MiniCssExtractPlugin({ filename: "[contenthash].css" }),
    new HtmlWebpackPlugin({
      filename: "index.html",
      title: info.TITLE,
      chunks: ["main"],
      template: "./src/index.html",
      templateParameters: { TITLE: info.TITLE },
    }),
  ],
};

module.exports = [config];
